//*****************************************************************************
//! msp_wm430fr4_golbal.h
//
//! Function Declarations.
//
//! \author Texas Instruments
//! \author A0222632
//
//! \version Release 1
//
//*****************************************************************************

#ifndef MSP_WM430FR4_GOLBAL_H_
#define MSP_WM430FR4_GOLBAL_H_

#include <msp430.h>
#include <stdint.h>
#include <stdbool.h>
#include <User_Config.h>
#include <msp_wm430fr4_lcd.h>
#ifdef Comm_RS485
#include <msp_wm430fr4_rs485.h>
#endif
#ifdef Comm_NFC
#include <msp_wm430fr4_nfc.h>
#endif
#ifdef Comm_Sub1GHz
#include <msp_wm430fr4_cc1120.h>
#endif

// Fram record address
#define FRAM_RECORD_ADDRESS 0xFF00	 // Refresh every 1 second
#define FRAM_RECORD_CHECK 0xFF40	 // First record check address
#define FRAM_RECORD_BASE 0xFA00	     // Monthly record in stand-alone address

// LED port definition
#define LED_PORT_OUT	P2OUT
#define LED_PORT_DIR	P2DIR
#define LED_PORT_SEL	P2SEL0
#define LED_BIT			BIT4
#define LED_ON()		(LED_PORT_OUT |= LED_BIT)
#define LED_OFF()		(LED_PORT_OUT &=~ LED_BIT)
#define LED_TOGGLE()    (LED_PORT_OUT ^= LED_BIT)

// Buzzer port definition
#define BUZZER_PORT_OUT	P4OUT
#define BUZZER_PORT_DIR	P4DIR
#define BUZZER_PORT_SEL	P4SEL0
#define	BUZZER_BIT		BIT0
#define	BUZZER_ON()		(BUZZER_PORT_OUT &= ~BUZZER_BIT)
#define	BUZZER_OFF()	(BUZZER_PORT_OUT |= BUZZER_BIT)

// VALVE port definition
#define VALVE_PORT_OUT1	P1OUT
#define VALVE_PORT_OUT2	P8OUT
#define	VALVE_PORT_IN	P1IN
#define VALVE_PORT_DIR	P1DIR
#define	VALVE_PORT_SEL	P1SEL0
#define	VALVE_PORT_REN	P1REN
#define	VALVE_PORT_IFG	P1IFG
#define	VALVE_PORT_IES	P1IES
#define	VALVE_PORT_IE	P1IE
#define	VALVE_BIT_IN1	BIT0
#define	VALVE_BIT_IN2	BIT1
#define	VALVE_BIT_OPEN	BIT3
#define	VALVE_BIT_CLOSE	BIT4
#define VALVE_ENABLE()	(VALVE_PORT_OUT2 |= BIT3)
#define VALVE_DISABLE()	(VALVE_PORT_OUT2 &=~ BIT3)
#define	IS_VALVE_OPEN	(VALVE_PORT_IN & VALVE_BIT_OPEN)
#define	IS_VALVE_CLOSE	(VALVE_PORT_IN & VALVE_BIT_CLOSE)
#define OPEN_VALVE()	{VALVE_ENABLE(); VALVE_PORT_OUT2 &=~ VALVE_BIT_IN2; VALVE_PORT_OUT2 |= VALVE_BIT_IN1;}
#define CLOSE_VALVE()	{VALVE_ENABLE(); VALVE_PORT_OUT2 &=~ VALVE_BIT_IN1; VALVE_PORT_OUT2 |= VALVE_BIT_IN2;}
#define	STOP_VALVE()	{VALVE_PORT_OUT2 &=~ (VALVE_BIT_IN1 + VALVE_BIT_IN2);}

// RTC format structure
typedef struct
{
	uint8_t ui8Year;
	uint8_t ui8Month;
	uint8_t ui8Day;
	uint8_t ui8Hour;
	uint8_t ui8Minute;
	uint8_t ui8Second;
} tWMRTC;


extern uint32_t ui32Sum;
extern uint16_t ui16Freq;
extern tWMRTC tRTCData;
extern uint8_t ui8RX_finished;

extern bool bOpen_Valve_Flag;
extern bool bClose_Valve_Flag;

extern void WM_Initialize(void);
extern void Measure_standby();
extern void FRAMWrite_Long(uint32_t *pFRAM_write_ptr, uint32_t ui32Data);
extern void FRAMWrite_Char(uint8_t *pFRAM_write_ptr, uint8_t ui8Data);

extern void NFC_Tag_Update(uint16_t ui16No, uint32_t ui32Sum, tWMRTC tRTCData);

#endif /* MSP_WM430FR4_GOLBAL_H_ */
