//*****************************************************************************
//! msp_wm430fr4_nfc.h
//
//! Function Declarations.
//
//! \author Texas Instruments
//! \author A0222632
//
//! \version Release 1
//
//*****************************************************************************

#include <msp_wm430fr4_golbal.h>

#ifndef MWP_WM430FR4_NFC_H_
#define MWP_WM430FR4_NFC_H_

// IIC port definition
#define PORT_I2C_OUT	P5OUT
#define PORT_I2C_DIR	P5DIR
#define PORT_I2C_SEL	P5SEL0
#define	BIT_SDA			BIT2
#define	BIT_SCL			BIT3

// INTO
#define PORT_INTO_IN	P1IN
#define PORT_INTO_OUT	P1OUT
#define PORT_INTO_DIR	P1DIR
#define PORT_INTO_SEL	P1SEL0
#define PORT_INTO_REN	P1REN
#define PORT_INTO_IE	P1IE
#define PORT_INTO_IES	P1IES
#define PORT_INTO_IFG	P1IFG
#define BIT_INTO		BIT2

// RST
#define PORT_RST_OUT	P5OUT
#define PORT_RST_DIR	P5DIR
#define PORT_RST_SEL	P5SEL0
#define BIT_RST			BIT4

//define the values for Granite's registers we want to access
#define CONTROL_REG 		0xFFFE
#define STATUS_REG			0xFFFC
#define INT_ENABLE_REG		0xFFFA
#define INT_FLAG_REG		0xFFF8
#define CRC_RESULT_REG		0xFFF6
#define CRC_LENGTH_REG		0xFFF4
#define CRC_START_ADDR_REG	0xFFF2
#define COMM_WD_CTRL_REG	0xFFF0
#define VERSION_REG			0xFFEE //contains the software version of the ROM
#define TEST_FUNCTION_REG   0xFFE2
#define TEST_MODE_REG		0xFFE0

//define the different virtual register bits
//CONTROL_REG bits
#define SW_RESET		BIT0
#define RF_ENABLE		BIT1
#define INT_ENABLE		BIT2
#define INTO_HIGH		BIT3
#define INTO_DRIVE		BIT4
#define BIP8_ENABLE		BIT5
#define STANDBY_ENABLE	BIT6
#define TEST430_ENABLE	BIT7

//STATUS_REG bits
#define READY			BIT0
#define CRC_ACTIVE		BIT1
#define RF_BUSY			BIT2

//INT_ENABLE_REG bits
#define EOR_INT_ENABLE		BIT1
#define EOW_INT_ENABLE		BIT2
#define CRC_INT_ENABLE		BIT3
#define BIP8_ERROR_INT_ENABLE		BIT4
#define NDEF_ERROR_INT_ENABLE		BIT5
#define GENERIC_ERROR_INT_ENABLE	BIT7

//INT_FLAG_REG bits
#define EOR_INT_FLAG	BIT1
#define EOW_INT_FLAG	BIT2
#define CRC_INT_FLAG	BIT3
#define BIP8_ERROR_INT_FLAG	BIT4
#define NDEF_ERROR_INT_FLAG	BIT5
#define GENERIC_ERROR_INT_FLAG	BIT7

//COMM_WD_CTRL_REG bits
#define WD_ENABLE	BIT0
#define TIMEOUT_PERIOD_2_SEC	0
#define TIMEOUT_PERIOD_32_SEC	BIT1
#define TIMEOUT_PERIOD_8_5_MIN	BIT2
#define TIMEOUT_PERIOD_MASK		BIT1 + BIT2 + BIT3

#define TEST_MODE_KEY 0x004E

#define RF430_DEFAULT_DATA		{  														\
/*NDEF Tag Application Name*/ 															\
0xD2, 0x76, 0x00, 0x00, 0x85, 0x01, 0x01, 												\
																						\
/*Capability Container ID*/ 															\
0xE1, 0x03, 																			\
0x00, 0x0F,	/* CCLEN */																	\
0x20,		/* Mapping version 2.0 */													\
0x00, 0x3B,	/* MLe (49 bytes); Maximum R-APDU data size */								\
0x00, 0x34, /* MLc (52 bytes); Maximum C-APDU data size */								\
0x04, 		/* Tag, File Control TLV (4 = NDEF file) */									\
0x06, 		/* Length, File Control TLV (6 = 6 bytes of data for this tag) */			\
0xE1, 0x04,	/* File Identifier */														\
0x0B, 0xDF, /* Max NDEF size (3037 bytes of useable memory) */							\
0x00, 		/* NDEF file read access condition, read access without any security */		\
0x00, 		/* NDEF file write access condition; write access without any security */	\
																						\
/* NDEF File ID */ 																		\
0xE1, 0x04, 																			\
																						\
/* NDEF File for Hello World  (76 bytes total length) */								\
0x00, 0x30, /* NLEN; NDEF length (3 byte long message) */ 								\
0xD1, 0x01, 0x2C, 																		\
0x54, /* T = text */																	\
0x02, 																					\
0x65, 0x6E, /* 'e', 'n', */																\
																						\
/* 'Hello, world!' NDEF data; Empty NDEF message, length should match NLEN*/			\
0x48, 0x65, 0x6C, 0x6C, 0x6f, 0x2c, 0x20, 0x77, 0x6f, 0x72, 0x6c, 0x64, 0x21, 0x0A,		\
0x48, 0x65, 0x6C, 0x6C, 0x6f, 0x2c, 0x20, 0x77, 0x6f, 0x72, 0x6c, 0x64, 0x21, 0x0A,		\
0x48, 0x65, 0x6C, 0x6C, 0x6f, 0x2c, 0x20, 0x77, 0x6f, 0x72, 0x6c, 0x64, 0x21 			\
}

uint16_t I2C_Read_Register(uint16_t ui16Address);
void I2C_Read_Continuous(uint16_t ui16Address, uint8_t* pRead_data, uint16_t ui16Data_length);

void I2C_Write_Register(uint16_t ui16Address, uint16_t ui16Value);
void I2C_Write_Continuous(uint16_t ui16Address, uint8_t* pWrite_data, uint16_t ui16Data_length);

extern void HAL_NFC_INIT();
extern void NFC_PROCESS();

#endif /* MWP_WM430FR4_NFC_H_ */
