//*****************************************************************************
//! msp_wm430fr4_rs485.h
//
//! Function Declarations.
//
//! \author Texas Instruments
//! \author A0222632
//
//! \version Release 1
//
//*****************************************************************************
#ifndef MSP_WM430FR4_COMM_H_
#define MSP_WM430FR4_COMM_H_

// UART Port definition
#define UART_PORT_OUT	P1OUT
#define UART_PORT_DIR	P1DIR
#define	UART_PORT_SEL	P1SEL0
#define	UART_BIT_TXD	BIT0
#define	UART_BIT_RXD	BIT1

// Meter type
#define	WM_Cold 		(0x10)
#define	WM_Domestic 	(0x11)
#define	WM_Drinking		(0x12)

// AMR Command
#define CMD_ReadData	(0x01)	// 0000 0001
#define CMD_WriteData	(0x04)	// 0000 0100
#define	CMD_ReadKeyVer	(0x09)	// 0000 1001
#define	CMD_ReadAddr	(0x03)	// 0000 0011
#define CMD_WriteAddr	(0x15)	// 0001 0101
#define CMD_Calibrate	(0x16)	// 0001 0110
#define CMD_ReadRTC		(0x21)	// 0010 0001
#define	CMD_WriteRTC	(0x22)	// 0010 0010
#define	CMD_OpenValve	(0x23)	// 0010 0011
#define	CMD_CloseValve	(0x24)	// 0010 0100

// Error ID
#define Error_SOF		(0x01)
#define Error_MT		(0x02)
#define Error_Address	(0x04)
#define Error_CMD		(0x08)

// Meter Address structure
typedef struct
{
	uint8_t ui8Addr0;
	uint8_t ui8Addr1;
	uint8_t ui8Addr2;
	uint8_t ui8Addr3;
	uint8_t ui8Addr4;
	uint8_t ui8Addr5;
	uint8_t ui8Addr6;
}tWMSN;

//*****************************************************************************
//
//! \typedef tParameterPacket is a union structure that stores a AMR
//! parameter packet.  The union allows the low-level serial driver to
//! interface with a buffer, while higher-level applications can access
//! specific portions of the packet.
//
//*****************************************************************************
typedef struct
{
	uint8_t ui8SOF;
	uint8_t ui8MeterType;
	tWMSN pWMAddr;
	uint8_t ui8Command;
	uint8_t ui8DataLength;
	uint8_t Data[12];
	uint8_t ui8Checksum;
	uint8_t ui8EOF;
}tAMRPacket;

extern uint8_t Uart_DataProcess();
extern void HAL_RS485_INIT();
extern bool bRS485_Send_Flag;
extern bool bRX_finished;

#endif /* MSP_WM430FR4_COMM_H_ */
