/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

//*****************************************************************************
//
//! TIDM-FRAM-WATERMETER (Hardware Revision 1.0, Jul. 2014)
//! main.c
//
//! This is
//! TIDM-FRAM-WATERMETER reference board.
//
//! \author Texas Instruments
//! \author A0222632
//
//! \version Release 1, 7/2014
//
//*****************************************************************************

#include <msp_wm430fr4_golbal.h>

uint16_t ui16MeterNum = Meter_Number;
uint32_t ui32Sum = 0;	                 // Total
uint16_t ui16Freq = 0;	                 // Instantaneous
uint8_t ui8NoWorking_Cnt = 0;
bool bWorking_flag = true;
bool bADC_done_flag = false;
bool bOpen_Valve_Flag = false;
bool bClose_Valve_Flag = false;

#ifdef Comm_RS485
//bool bRS485_Send_Flag = false;
#endif
#ifdef Comm_NFC
bool bInto_Fired = false;
#endif
#ifdef Comm_Sub1GHz
bool bCC1120_Send_Flag = false;
#define PKTLEN	30
uint8_t ui8packetSemaphore;
uint16_t ui16packetCounter;
// Initialize packet buffer of size PKTLEN + 1
uint8_t ui8TxBuffer[PKTLEN + 1] = { 0 };
#endif

// For ADC
uint32_t ui32Bat_Vol;

void main(void)
{
	WM_Initialize();                // System initialize

//	SFRRPCR |= SYSNMI;              // Select NMI mode

#ifdef Comm_RS485
	HAL_RS485_INIT();
#endif
#ifdef Comm_NFC
	HAL_NFC_INIT();
#endif
#ifdef Comm_Sub1GHz
	HAL_CC1120_TX_INIT();
	SPI_Standby();
#endif

	// Check battery voltage, if Voltage less than 2.8V then stop here
	do{
		ADCCTL0 |= ADCENC + ADCSC;
		while(ADCCTL1 & ADCBUSY);
		ui32Bat_Vol = (uint32_t)150 * 1023 / (ADCMEM0&0x3FF);
		LCD_Display_BatteryVolgate(ui32Bat_Vol);
	}while(ui32Bat_Vol < 280);

	LCD_Display_Error(0);   // For test

	// Check if the first record or not
	if((*((uint32_t *)FRAM_RECORD_CHECK)) == 0xAA)
	{
		ui32Sum = *((uint32_t *)FRAM_RECORD_ADDRESS); // Load the last data
	}
	else
	{
		ui32Sum = 0;
		FRAMWrite_Long((uint32_t *)FRAM_RECORD_CHECK,0xAA);
	}

	LED_OFF();
	BUZZER_OFF();

	while(1)
	{
		__bis_SR_register(LPM3_bits + GIE);   // Enter LPM3 w/ interrupt

		if(bWorking_flag)
		{
			//every 1 second
			ui16Freq = TA0R;
			ui32Sum += ui16Freq;
			TA0CTL = TASSEL_0 + MC_2 + TACLR;
			FRAMWrite_Long((uint32_t *)FRAM_RECORD_ADDRESS,ui32Sum); //write to FRAM (refresh)
#ifdef Comm_NFC
			NFC_Tag_Update(ui16MeterNum,ui32Sum,tRTCData);
#endif
			if(bADC_done_flag)
			{
				LCD_Display_BatteryVolgate(ui32Bat_Vol);
				bADC_done_flag = false;
				// while(Bat_Vol < 280);       // Battery low action. To do ...
				// if(Bat_Vol < 280) bClose_Valve_Flag = true; // If battery low, close the valve
			}
			else
			{
				LCD_DisplayLongDecimal_A(ui32Sum,3);
				LCD_DisplayLongDecimal_B(ui16Freq,3);
			}

			if(ui16Freq == 0)
			{
				if(ui8NoWorking_Cnt > 5)
				{
					bWorking_flag = false;
					ui8NoWorking_Cnt = 0;
					TA0CTL = 0;
					P1SEL0 &= ~BIT5;
					P1DIR &= ~BIT5;
//					P1OUT |= BIT5;		// to be comment
//					P1REN |= BIT5;		// to be comment
					P1IFG &= ~BIT5;
					P1IES |= BIT5;
					P1IE |= BIT5;
				}
				else ui8NoWorking_Cnt++;
			}
		}
#ifdef Comm_NFC
		if(bInto_Fired)
		{
			NFC_PROCESS();
			bInto_Fired = false;
		}
#endif
#ifdef Comm_Sub1GHz
		if(bCC1120_Send_Flag)
		{
			SPI_Config();

			// update packet counter
			ui16packetCounter++;

			// create a random packet with PKTLEN + 2 byte packet counter + n x random bytes
			ui8TxBuffer[0] = PKTLEN; // Length byte
			ui8TxBuffer[1] = (unsigned char) ui16packetCounter >> 8; // MSB of ui16packetCounter
			ui8TxBuffer[2] = (unsigned char) ui16packetCounter; // LSB of ui16packetCounter

			ui8TxBuffer[3] = ui16MeterNum;
			ui8TxBuffer[4] = (unsigned char) ((ui32Sum&0xFF000000)>>24);
			ui8TxBuffer[5] = (unsigned char) ((ui32Sum&0x00FF0000)>>16);
			ui8TxBuffer[6] = (unsigned char) ((ui32Sum&0x0000FF00)>>8);
			ui8TxBuffer[7] = (unsigned char) (ui32Sum&0x000000FF);

			ui8TxBuffer[10] = tRTCData.ui8Year;
			ui8TxBuffer[11] = tRTCData.ui8Month;
			ui8TxBuffer[12] = tRTCData.ui8Day;
			ui8TxBuffer[13] = tRTCData.ui8Hour;
			ui8TxBuffer[14] = tRTCData.ui8Minute;
			ui8TxBuffer[15] = tRTCData.ui8Second;

			// write packet to tx fifo
			cc112xSpiWriteTxFifo(ui8TxBuffer, sizeof(ui8TxBuffer));

			// strobe TX to send packet
			TrxSpiCmdStrobe(CC112X_STX);

			// wait for interrupt that packet has been sent.
			// (Assumes the GPIO connected to the radioRxTxISR function is set
			// to GPIOx_CFG = 0x06)
			while (!ui8packetSemaphore);

			// clear semaphore flag
			ui8packetSemaphore = 0;

			bCC1120_Send_Flag = false;

			TrxSpiCmdStrobe(CC112X_SPWD);

			SPI_Standby();
		}
#endif
#ifdef Comm_RS485
		if(bRX_finished)
		{
			// rs485
			Uart_DataProcess();
			bRX_finished = false;
		}
#endif
		if(bOpen_Valve_Flag)
		{
			if(IS_VALVE_CLOSE)// if not open
			{
				OPEN_VALVE();
			}
		}
		if(bClose_Valve_Flag)
		{
			if(IS_VALVE_OPEN)// if not close
			{
				CLOSE_VALVE();
			}
		}

	}
}

#pragma vector=PORT1_VECTOR
__interrupt void PORT1_ISR(void)
{
#ifdef Comm_NFC
	if(P1IFG & BIT_INTO)
	{
		bInto_Fired = true;
		P1IE &= ~BIT_INTO; //disable INTO
		P1IFG &= ~BIT_INTO; //clear interrupt flag
	}
#endif
	if(P1IFG & BIT3)
	{
		bClose_Valve_Flag = false;
		STOP_VALVE();
		VALVE_DISABLE();
		P1IFG &= ~BIT3;
	}

	if(P1IFG & BIT4)
	{
		bOpen_Valve_Flag = false;
		STOP_VALVE();
		VALVE_DISABLE();
		P1IFG &= ~BIT4;
	}

	if(P1IFG & BIT5)
	{
		bWorking_flag = true;
		ui8NoWorking_Cnt = 0;

		P1IES &=~ BIT5;
		P1IE &=~ BIT5;
//		P1REN &= ~BIT5; // to be comment
		P1SEL0 |= BIT5;
		P1IFG &=~ BIT5;
	}
}

#pragma vector=PORT2_VECTOR
__interrupt void port2_ISR(void)
{
#ifdef Comm_Sub1GHz
	if (P2IFG & BIT6)
	{
		// set packet semaphore
		ui8packetSemaphore = 1;
		// clear isr flag
		P2IFG &= ~BIT6;
	}
#endif
}

#define CALADC_15V_30C  *((unsigned int *)0x1A1A)               // Temperature Sensor Calibration-30 C
                                                                // See device datasheet for TLV table memory mapping
#define CALADC_15V_85C  *((unsigned int *)0x1A1C)               // Temperature Sensor Calibration-85 C

#pragma vector=ADC_VECTOR
__interrupt void ADC10_ISR(void)
{
	switch(__even_in_range(ADCIV,12))
	{
	case 0: break;    // No interrupt
	case 2: break;    // conversion result overflow
	case 4: break;    // conversion time overflow
	case 6: break;    // ADC10HI
	case 8: break;    // ADC10LO
	case 10: break;   // ADC10IN
	case 12:
		ui32Bat_Vol = (uint32_t)150 * 1023 / (ADCMEM0&0x3FF); //100
		bADC_done_flag = true;
		PMMCTL2 &= ~INTREFEN;   // For low power
		ADCCTL0 &= ~ADCON;
//		if(ui32Bat_Vol < 280)
//		{
//			// Battery low action. To do ...
//		}
		break;
	default: break;
	}
}
