//*****************************************************************************
//! msp_wm430fr4_golbal.c
//
//!
//
//! \author Texas Instruments
//! \author A0222632
//
//! \version Release 1
//
//*****************************************************************************

#include <msp_wm430fr4_golbal.h>

uint8_t const ui8Month_Table[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
tWMRTC tRTCData;

//*****************************************************************************
//
//! Perform GPIO initialize.
//!
//
//! \no param
//!
//! \return None.
//
//*****************************************************************************
void GPIO_Config(void)
{
	P1OUT &= ~(BIT0 + BIT1 + BIT2 + BIT3 + BIT4 + BIT5 + BIT6 + BIT7);
	P1DIR |= BIT0 + BIT1 + BIT2 + BIT3 + BIT4 + BIT5 + BIT6 + BIT7;
	P2OUT &= ~(BIT4 + BIT5 + BIT6 + BIT7);
	P2DIR |= BIT4 + BIT5 + BIT6 + BIT7;
	P4OUT &= ~BIT0;
	P4DIR |= BIT0;
	P5OUT &= ~(BIT0 + BIT1 + BIT2 + BIT3 + BIT4 + BIT5 + BIT6 + BIT7);
	P5DIR |= BIT0 + BIT1 + BIT2 + BIT3 + BIT4 + BIT5 + BIT6 + BIT7;
	P8OUT &= ~(BIT0 + BIT1 + BIT2 + BIT3);
	P8DIR |= BIT0 + BIT1 + BIT2 + BIT3;
}

//*****************************************************************************
//
//! Perform System clock initialize.
//! ACLK = XT1CLK = 32.738kHz  MCLK = SMCLK = default ~ 1.04MHz
//
//! \no param
//!
//! \return None.
//
//*****************************************************************************
void CLOCK_Config()
{
	P4SEL0 |= BIT1 + BIT2;                  // Select pin4.1 pin4.2 as crystal function
	do
	{
		CSCTL7 &= ~(XT1OFFG + DCOFFG);    	// Clear XT1 and DCO fault flag
		SFRIFG1 &= ~OFIFG;
	}while (SFRIFG1 & OFIFG);             	// Test oscillator fault flag

	CSCTL4 = SELA__XT1CLK;					// IMPORTANT. Select ACLK = XT1 = 32768Hz
}

//*****************************************************************************
//
//! Perform LCD initialize.
//!
//
//! \no param
//!
//! \return None.
//
//*****************************************************************************
void LCD_Config()
{
	uint8_t ui8Cnt;
	uint8_t *pLCD = (uint8_t *)&LCDM0;
	for (ui8Cnt = 14; ui8Cnt > 0; ui8Cnt--)			// Clear LCD memory
		*pLCD++ = 0x00;					// CLR LCD

	LCDM0 = BIT7;						// COM0 -- L21
	LCDM1 = BIT6;						// COM1 -- L23
	LCDM2 = BIT5;						// COM2 -- L25
	LCDM3 = BIT4;						// COM3 -- L27

	SYSCFG2 |= LCDPCTL;      			// R13/R23/R33/LCDC1/LCDC2 pins selected

	LCDPCTL0 = 0xFFFF;
	LCDPCTL1 = 0x0FFF;					// L0~L27 pins selected

	LCDVCTL = LCDSELVDD + LCDCPEN;		// Mode 2

	LCDCSSEL0 = 0x00AA;					// Configure COMs and SEGs
	LCDCSSEL1 = 0x0000;

	LCDCTL0 |= LCD4MUX + LCDLP + LCDON;	// turn on LCD: 4-mux selected
}

//*****************************************************************************
//
//! Perform ADC initialize.
//! Clock source MODCLK / Channel A13 Internal reference 1.5V
//! Reference AVCC+ AVSS- / 50ksps
//! 10-bit resolution
//
//! \no param
//!
//! \return None.
//
//*****************************************************************************
void ADC10_Config()
{
	CSCTL8 |= MODCLKREQEN;
	PMMCTL0 = SVSHE + PMMPW;
	PMMCTL2 |= INTREFEN;                // 1.5V voltage reference generate
	ADCCTL0 = ADCSHT_8 + ADCON;         // 256 ADCCLK cycles
	ADCCTL1 = ADCSHS_0 + ADCSHP + ADCSSEL_0 + ADCCONSEQ_0;  // ADCSC bit SHS,
	// MODCLK
	// SC-SC
	ADCCTL2 |= ADCRES + ADCSR;          // 10 bit resolution, 50ksps
	ADCMCTL0 = ADCSREF_0 + ADCINCH_13;  // Reference AVCC+/AVSS-, Channel 13
	ADCIE |= ADCIE0;                    // Interrupt enable

	__delay_cycles(400);
}

//*****************************************************************************
//
//! Perform RTC initialize.
//! Clock source XT1CLK
//
//! \no param
//!
//! \return None.
//
//*****************************************************************************
void RTC_Config()
{
	RTCCTL = RTCSS__XT1CLK + RTCSR + RTCPS__1 | RTCIE;
	RTCMOD = 32768-1;		             //1s
	tRTCData.ui8Year = 14;              // RTC initialize
	tRTCData.ui8Month = 7;
	tRTCData.ui8Day = 30;
	tRTCData.ui8Hour = 23;
	tRTCData.ui8Minute = 59;
	tRTCData.ui8Second = 55;
}

//*****************************************************************************
//
//! Perform Timer configuration.
//!
//
//! \param
//!
//! \return None.
//
//*****************************************************************************
void Valve_Config()
{
	// DRV8837
	VALVE_PORT_OUT2 &=~ (VALVE_BIT_IN1 | VALVE_BIT_IN2);

	// SENSOR-position detect
	VALVE_PORT_DIR &= ~(VALVE_BIT_OPEN | VALVE_BIT_CLOSE);
	VALVE_PORT_OUT1 |= VALVE_BIT_OPEN | VALVE_BIT_CLOSE;
	VALVE_PORT_REN |= VALVE_BIT_OPEN | VALVE_BIT_CLOSE;
	VALVE_PORT_IFG &= ~(VALVE_BIT_OPEN | VALVE_BIT_CLOSE);
	VALVE_PORT_IES |= VALVE_BIT_OPEN | VALVE_BIT_CLOSE;
	VALVE_PORT_IE |= VALVE_BIT_OPEN | VALVE_BIT_CLOSE;
	VALVE_DISABLE();
}

//*****************************************************************************
//
//! Perform Water Meter board initialize.
//!
//
//! \no param
//!
//! \return None.
//
//*****************************************************************************
void WM_Initialize(void)
{
	WDTCTL = WDTPW + WDTHOLD;  	// Stop watchdog
	//	WDTCTL = WDTPW + WDTCNTCL + WDTSSEL0 + WDTIS0 + WDTIS1;  // Set watchdog 16s

	PM5CTL0 &= ~LOCKLPM5;      	// Disable the GPIO power-on default high-impedance mode
								// to activate previously configured port settings
	GPIO_Config();              // GPIO initialize
	LED_ON();
	BUZZER_ON();
	CLOCK_Config();             // System clock initialize
	LCD_Config();               // LCD module initialize
	RTC_Config();
	ADC10_Config();
	Valve_Config();
	P1SEL0 |= BIT5;
//	P1REN |= BIT5;
	P1REN &= ~BIT5; // to be comment
	TA0CTL = 0;
}

void FRAMWrite_Long(uint32_t *pFRAM_write_ptr, uint32_t ui32data)
{
	SYSCFG0 &= ~PFWP;
	*pFRAM_write_ptr = ui32data;
	SYSCFG0 |= PFWP;
}

void FRAMWrite_Char(uint8_t *pFRAM_write_ptr, uint8_t ui8data)
{
	SYSCFG0 &= ~PFWP;
	*pFRAM_write_ptr = ui8data;
	SYSCFG0 |= PFWP;
}

// RTC interrupt service routine
#pragma vector=RTC_VECTOR
__interrupt void RTC_ISR(void)
{
	switch(__even_in_range(RTCIV,RTCIV_RTCIF))
	{
	case RTCIV_NONE: break;             // No interrupt

	case RTCIV_RTCIF:                   // RTC Overflow
	{
		uint8_t ui8Days = 31;
		uint8_t *pAddress;               // For monthly data storage

		tRTCData.ui8Second++;

		if(tRTCData.ui8Second == 59)
		{
			PMMCTL2 |= INTREFEN;                // 1.5V voltage reference generate
			ADCCTL0 = ADCSHT_8 + ADCON;         // 256 ADCCLK cycles
		}
		if (tRTCData.ui8Second >= 60)       // 60s
		{
#ifdef Comm_Sub1GHz
			bCC1120_Send_Flag = true;  // send the data via wireless (move to monthly)
#endif
			ADCCTL0 |= ADCENC + ADCSC;  // Battery voltage detect each 1 minute
			tRTCData.ui8Second = 0;
			tRTCData.ui8Minute++;
			if (tRTCData.ui8Minute >= 60)
			{
				tRTCData.ui8Minute = 0;
				tRTCData.ui8Hour++;
			}

			if (tRTCData.ui8Hour >= 24)
			{
				tRTCData.ui8Hour = 0;
				tRTCData.ui8Day++;
				if (tRTCData.ui8Month == 2)
				{
					if (tRTCData.ui8Year % 4 == 0)
					{
						ui8Days = 29;
					}
					else
					{
						ui8Days = 28;
					}
				}
				else
				{
					ui8Days = ui8Month_Table[tRTCData.ui8Month - 1];
				}

			}

			if (tRTCData.ui8Day > ui8Days)
			{
				// Monthly record
				pAddress = (uint8_t *)(FRAM_RECORD_BASE + (tRTCData.ui8Year - 13)*120 + (tRTCData.ui8Month - 1)*10);
				FRAMWrite_Char(pAddress++,tRTCData.ui8Year);
				FRAMWrite_Char(pAddress++,tRTCData.ui8Month);
				FRAMWrite_Char(pAddress++,tRTCData.ui8Day-1);
				FRAMWrite_Char(pAddress++,tRTCData.ui8Hour);
				FRAMWrite_Char(pAddress++,tRTCData.ui8Minute);
				FRAMWrite_Char(pAddress++,tRTCData.ui8Second);
				FRAMWrite_Long((uint32_t *)(pAddress++),ui32Sum);

				tRTCData.ui8Day = 1;
				tRTCData.ui8Month++;
			}

			if (tRTCData.ui8Month > 12)
			{
				tRTCData.ui8Month = 1;
				tRTCData.ui8Year++;
			}
		}
		__low_power_mode_off_on_exit();
	}
	break;
	default: break;
	}
}
