//*****************************************************************************
//! msp_wm430fr4_lcd.c
//
//! Functions to support lcd display.  Some
//! functions are a direct access of the LCD_E registers while others take
//! the definitions (see msp_wm430fr4_lcd.h) and place the approriate values in the
//! LCD_E peripheral.
//
//! \author Texas Instruments
//! \author A0222632
//
//! \version Release 1
//
// ***************************************************
// *                --   --   --   --                *
// *               |  | |  | |  | |  |               *
// *                --   --   --   --                *
// *               |  | |  | |  | |  |               *
// *                --   --   --   --                *
// *      --   --   --   --   --   --   --   --      *
// *     |  | |  | |  | |  | |  | |  | |  | |  |     *
// *      --   --   --   --   --   --   --   --      *
// *     |  | |  | |  | |  | |  | |  | |  | |  |     *
// *      --   --   --   --   --   --   --   --      *
// ***************************************************
//
//*****************************************************************************
#include <msp_wm430fr4_golbal.h>

#define a       0x01                            //  AAAA
#define b       0x02                            // F    B
#define c       0x04                            // F    B
#define DOTSEG  0x08                            //  GGGG
#define f  		0x10                            // E    C
#define g       0x20                            // E    C
#define e       0x40                            //  DDDD
#define d       0x80
#define NEGSEG  0x20

const uint8_t LCD_Tab[] = {
		a + b + c + d + e + f,			// Displays "0"
		b + c,							// Displays "1"
		a + b + d + e + g,				// Displays "2"
		a + b + c + d + g,				// Displays "3"
		b + c + f + g,					// Displays "4"
		a + c + d + f +g,				// Displays "5"
		a + c + d + e + f + g,			// Displays "6"
		a + b + c,						// Displays "7"
		a + b + c + d + e + f + g,		// Displays "8"
		a + b + c + d + f + g,			// Displays "9"
		a + b + c + e + f + g,			// Displays "A"
		c + d + e + f + g,				// Displays "B"
		a + d + e + f,					// Displays "C"
		b + c + d + e + g,				// Displays "D"
		a + d + e + f + g,				// Displays "E"
		a + e + f + g,					// Displays "F"
		a + c + d + e + f,				// Displays "G"
		b + c + e + f + g,				// Displays "H"
		e + f,							// Displays "I"
		b + c + d + e,					// Displays "J"
		b + d + e + f + g,				// Displays "K"
		d + e + f,						// Displays "L"
		a + c + e + g,					// Displays "M"
		a + b + c + e + f,				// Displays "N"
		c + e + g,						// Displays "n"
		c + d + e + g,					// Displays "o"
		a + b + c + d + e + f,			// Displays "O"
		a + b + e + f + g,				// Displays "P"
		a + b + c + f + g,				// Displays "Q"
		e + g,							// Displays "r"
		a + c + d + f +g,				// Displays "S"
		d + e + f + g,					// Displays "t"
		a + e + f,						// Displays "T"
		b + c + d + e + f,				// Displays "U"
		c + d + e,						// Displays "v"
		b + d + f + g,					// Displays "W"
		b + c + d + f + g,				// Displays "Y"
		a + b + d + e + g,				// Displays "Z"
		g,								// Displays "-"
		b,								// Displays "'"
		0								// Displays " "
};
#undef a
#undef b
#undef c
#undef d
#undef e
#undef f
#undef g

#define AA 10
#define BB AA+1
#define CC BB+1
#define DD CC+1
#define EE DD+1
#define FF EE+1
#define GG FF+1
#define HH GG+1
#define II HH+1
#define JJ II+1
#define KK JJ+1
#define LL KK+1
#define mm LL+1
#define NN mm+1
#define nn NN+1
#define oo nn+1
#define OO oo+1
#define PP OO+1
#define QQ PP+1
#define rr QQ+1
#define SS rr+1
#define tt SS+1
#define TT tt+1
#define UU TT+1
#define VV UU+1
#define WW VV+1
#define YY WW+1
#define ZZ YY+1
#define BR ZZ+1
#define DT BR+1
#define SP DT+1

//*****************************************************************************
//
//! Perform Number display.
//!
//
//! \param Number is , Position is
//!
//! \return None.
//
//*****************************************************************************
void LCD_DisplayNum(uint8_t Number, uint8_t Position)
{
	uint8_t *pLCD = (uint8_t *)&LCDM0;

	if(Position == 0)
	{
		pLCD[13] &= 0x0F;
		pLCD[13] |= (LCD_Tab[Number]&0x0F)<<4;
		pLCD[12] &= 0x0F;
		pLCD[12] |= LCD_Tab[Number]&0xF0;
	}
	else if((Position > 0) && (Position < 8))
	{
		pLCD[15-Position-Position] &= 0xF0;
		pLCD[15-Position-Position] |= (LCD_Tab[Number]&0xF0)>>4;
		pLCD[14-Position-Position] &= 0xF0;
		pLCD[14-Position-Position] |= LCD_Tab[Number]&0x0F;
	}
	else if((Position > 7) && (Position < 12))
	{
		pLCD[Position+Position-11] &= 0x0F;
		pLCD[Position+Position-11] |= (LCD_Tab[Number]&0x0F)<<4;
		pLCD[Position+Position-12] &= 0x0F;
		pLCD[Position+Position-12] |= LCD_Tab[Number]&0xF0;
	}
}

//*****************************************************************************
//
//! Perform Dot display.
//!
//
//! \param Position is
//!
//! \return None.
//
//*****************************************************************************
void LCD_DisplayDot(uint8_t Position)
{
	uint8_t *pLCD = (uint8_t *)&LCDM0;

	if(Position == 0)
	{
		pLCD[13] |= DOTSEG<<4;
	}
	else if((Position > 0) && (Position < 8))
	{
		pLCD[14-Position-Position] |= DOTSEG;

	}
	else if((Position > 7) && (Position < 12))
	{
		pLCD[Position+Position-11] |= DOTSEG<<4;
	}
}

//*****************************************************************************
//
//! Perform display Chinese character.
//!
//
//! \param
//!
//! \return None.
//
//*****************************************************************************
#define Display_Chinese_Character() (LCD_DisplayDot(0))

//*****************************************************************************
//
//! Perform second line display.
//!
//
//! \param
//!
//! \return None.
//
//*****************************************************************************
void LCD_DisplayLongDecimal_A(uint32_t ui32Number, uint8_t ui8Dot)
{
	uint8_t ui8Cnt;
	uint8_t DispBuff[3];

	for(ui8Cnt=9;ui8Cnt>0;ui8Cnt--)
	{
		DispBuff[9-ui8Cnt]=ui32Number%10;
		ui32Number/=10;
	}
	for(ui8Cnt=7;ui8Cnt>ui8Dot;ui8Cnt--)
	{
		if (DispBuff[ui8Cnt]==0) DispBuff[ui8Cnt]=SP;
		else break;
	}
	for(ui8Cnt=0;ui8Cnt<8;ui8Cnt++)
	{
		LCD_DisplayNum(DispBuff[ui8Cnt],ui8Cnt);
	}

	if(ui8Dot != 0) LCD_DisplayDot(ui8Dot);

#ifdef Chinese_Version
	Display_Chinese_Character();
#endif
}

//*****************************************************************************
//
//! Perform first line display.
//!
//
//! \param Number the number to be displayed; DOT the postion of dot
//!
//! \return None.
//
//*****************************************************************************
void LCD_DisplayLongDecimal_B(uint16_t ui16Number, uint8_t ui8Dot)
{
	uint8_t ui8Cnt;
	uint8_t DispBuff[4];

	for(ui8Cnt=0;ui8Cnt<4;ui8Cnt++)
	{
		DispBuff[ui8Cnt] = ui16Number%10;
		ui16Number /= 10;
	}
	for(ui8Cnt=3;ui8Cnt>ui8Dot;ui8Cnt--)
	{
		if (DispBuff[ui8Cnt]==0) DispBuff[ui8Cnt]=SP;
		else break;
	}
	for(ui8Cnt=8;ui8Cnt<12;ui8Cnt++)
	{
		LCD_DisplayNum(DispBuff[ui8Cnt-8],ui8Cnt);
	}
	LCD_DisplayDot(ui8Dot+8);
}

//*****************************************************************************
//
//! Perform display battery voltage informaiton.
//!
//
//! \param BalVol battery voltage to be displayed
//!
//! \return None.
//
//*****************************************************************************
void LCD_Display_BatteryVolgate(uint16_t ui16BatVol)
{
	LCD_DisplayLongDecimal_A((uint32_t)ui16BatVol*10,3);
	LCD_DisplayNum(VV,7);
	LCD_DisplayNum(oo,6);
	LCD_DisplayNum(LL,5);
	LCD_DisplayNum(SP,4);
	LCD_DisplayNum(VV,0);
	LCD_DisplayNum(SP,8);
	if(ui16BatVol < 280)
	{
		LCD_DisplayNum(LL,11);
		LCD_DisplayNum(oo,10);
		LCD_DisplayNum(WW,9);
	}
	else
	{
		LCD_DisplayNum(nn,11);
		LCD_DisplayNum(oo,10);
		LCD_DisplayNum(rr,9);
	}
}

//*****************************************************************************
//
//! Perform display system error informaiton.
//!
//
//! \param error ID to be displayed
//!
//! \return None.
//
//*****************************************************************************
void LCD_Display_Error(uint8_t ui8Error_ID)
{
	LCD_DisplayNum(BR,8);
	LCD_DisplayNum(BR,9);
	LCD_DisplayNum(BR,10);
	LCD_DisplayNum(BR,11);
	LCD_DisplayLongDecimal_A(ui8Error_ID,0);
	LCD_DisplayNum(EE,7);
	LCD_DisplayNum(rr,6);
	LCD_DisplayNum(rr,5);
	LCD_DisplayNum(oo,4);
	LCD_DisplayNum(rr,3);
}

//*****************************************************************************
//
//! Perform LCD display test.
//! First line "M430"
//! Second line "7654.3210"
//
//! \param None
//!
//! \return None.
//
//*****************************************************************************
void LCD_Disp_Test()
{
	LCD_DisplayNum(5,0);
	LCD_DisplayNum(mm,11);
	LCD_DisplayNum(4,10);
	LCD_DisplayNum(3,9);
	LCD_DisplayNum(0,8);
	LCD_DisplayLongDecimal_A(76543210,1);
	LCD_DisplayNum(0,7);
	LCD_DisplayNum(1,6);
	LCD_DisplayNum(2,5);
	LCD_DisplayNum(3,4);
	LCD_DisplayNum(4,3);
	LCD_DisplayNum(5,2);
	LCD_DisplayNum(6,1);
	LCD_DisplayNum(7,0);
}
