/*
 * msp_wm430fr4_nfc.c
 *
 *  Created on: 2014718
 *      Author: a0222632
 */


#include <msp_wm430fr4_golbal.h>

#ifdef Comm_NFC

uint8_t RxData[2] = {0,0};
uint8_t TxData[2] = {0,0};
uint8_t TxAddr[2] = {0,0};

uint8_t NDEF_Application_Data[] = RF430_DEFAULT_DATA;

// For NFC init
void eUSCIB0_Config()
{
    // Configure pins for I2C
    PORT_I2C_SEL |= BIT_SCL + BIT_SDA;

    //configure eUSCI for I2C
	UCB0CTL1 |= UCSWRST;	            			//Software reset enabled
	UCB0CTLW0 |= UCMODE_3  + UCMST + UCSYNC + UCTR;	//I2C mode, Master mode, sync, transmitter
	UCB0CTLW0 |= UCSSEL_2;                    		// SMCLK = 1.04MHz

	UCB0BRW = 80; 						// Baudrate = SMLK/6 = 100kHz

	UCB0I2CSA  = 0x0028;				//slave address - determined by pins E0, E1, and E2 on the RF430CL330H
	UCB0CTL1  &= ~UCSWRST;

	UCB0IFG  = 0xFFFF;
	UCB0CTL1  |= UCSWRST;
}

// Reads the register at reg_addr, returns the result
uint16_t I2C_Read_Register(uint16_t reg_addr)
{
	PORT_I2C_SEL |= BIT_SCL + BIT_SDA;

	TxAddr[0] = reg_addr >> 8; 		// MSB of address
	TxAddr[1] = reg_addr & 0xFF; 	// LSB of address

	UCB0CTLW1 = UCASTP_1;
	UCB0TBCNT = 0x0002;
	UCB0CTL1  &= ~UCSWRST;
	UCB0CTL1 |= UCTXSTT + UCTR;		// Start i2c write operation
	while(!(UCB0IFG & UCTXIFG0));
	UCB0TXBUF = TxAddr[0];
	while(!(UCB0IFG & UCTXIFG0));
	UCB0TXBUF = TxAddr[1];
	while(!(UCB0IFG & UCBCNTIFG));
	UCB0CTL1 &= ~UCTR; 				// I2C read operation
	UCB0CTL1 |= UCTXSTT; 			// Repeated start
	while(!(UCB0IFG & UCRXIFG0));
	RxData[0] = UCB0RXBUF;
	UCB0CTLW0 |= UCTXSTP; 			// Send stop after next RX
	while(!(UCB0IFG & UCRXIFG0));
	RxData[1] = UCB0RXBUF;
	while (!(UCB0IFG & UCSTPIFG));  // Ensure stop condition got sent
	UCB0CTL1  |= UCSWRST;

	PORT_I2C_SEL &= ~(BIT_SCL + BIT_SDA);
	PORT_I2C_OUT |= BIT_SCL + BIT_SDA;

	return RxData[1] << 8 | RxData[0];
}

//Continuous read data_length bytes and store in the area "read_data"
void I2C_Read_Continuous(uint16_t reg_addr, uint8_t* read_data, uint16_t data_length)
{
	uint16_t i;

	PORT_I2C_SEL |= BIT_SCL + BIT_SDA;

	TxAddr[0] = reg_addr >> 8; 		// MSB of address
	TxAddr[1] = reg_addr & 0xFF; 	// LSB of address


	UCB0CTLW1 = UCASTP_1;
	UCB0TBCNT = 0x0002;
	UCB0CTL1  &= ~UCSWRST;
	UCB0CTL1 |= UCTXSTT + UCTR;		// Start i2c write operation

	while(!(UCB0IFG & UCTXIFG0));
	UCB0TXBUF = TxAddr[0];
	while(!(UCB0IFG & UCTXIFG0));
	UCB0TXBUF = TxAddr[1];
	while(!(UCB0IFG & UCBCNTIFG));
	UCB0CTL1 &= ~UCTR; 			//i2c read operation
	UCB0CTL1 |= UCTXSTT; 		//repeated start
	while(!(UCB0IFG & UCRXIFG0));

	for(i = 0; i < data_length-1; i++)
	{
		while(!(UCB0IFG & UCRXIFG0));
		read_data[i] = UCB0RXBUF;
		if(i == data_length-1)
			UCB0CTL1 |= UCTXSTP; 	//send stop after next RX
	}

	UCB0CTLW0 |= UCTXSTP; 			//send stop after next RX
	while(!(UCB0IFG & UCRXIFG0));
	read_data[i] = UCB0RXBUF;
	while (!(UCB0IFG & UCSTPIFG)); 	// Ensure stop condition got sent
	UCB0CTL1  |= UCSWRST;

	PORT_I2C_SEL &= ~(BIT_SCL + BIT_SDA);
	PORT_I2C_OUT |= BIT_SCL + BIT_SDA;
}

//writes the register at reg_addr with value
void I2C_Write_Register(uint16_t reg_addr, uint16_t value)
{
	PORT_I2C_SEL |= BIT_SCL + BIT_SDA;

	TxAddr[0] = reg_addr >> 8; 		// MSB of address
	TxAddr[1] = reg_addr & 0xFF; 	// LSB of address
	TxData[0] = value >> 8;
	TxData[1] = value & 0xFF;

	UCB0CTLW1 = UCASTP_1;
	UCB0TBCNT = 0x0004;
	UCB0CTL1  &= ~UCSWRST;
	UCB0CTL1 |= UCTXSTT + UCTR;		// Start i2c write operation
	//write the address
	while(!(UCB0IFG & UCTXIFG0));
	UCB0TXBUF = TxAddr[0];
	while(!(UCB0IFG & UCTXIFG0));
	UCB0TXBUF = TxAddr[1];
	//write the data
	while(!(UCB0IFG & UCTXIFG0));
	//UCB0TXBUF = TxData[0];
	UCB0TXBUF = TxData[1];
	while(!(UCB0IFG & UCTXIFG0));
	//UCB0TXBUF = TxData[1];
	UCB0TXBUF = TxData[0];
	while (!(UCB0IFG & UCBCNTIFG));
	UCB0CTL1 |= UCTXSTP;
	while (!(UCB0IFG & UCSTPIFG)); 	// Ensure stop condition got sent
	UCB0CTL1  |= UCSWRST;

	PORT_I2C_SEL &= ~(BIT_SCL + BIT_SDA);
	PORT_I2C_OUT |= BIT_SCL + BIT_SDA;
}

//writes the register at reg_addr and incrementing addresses with the data at "write_data" of length data_length
void I2C_Write_Continuous(uint16_t reg_addr, uint8_t* write_data, uint16_t data_length)
{
	uint16_t i;

	PORT_I2C_SEL |= BIT_SCL + BIT_SDA;

	TxAddr[0] = reg_addr >> 8; 		//MSB of address
	TxAddr[1] = reg_addr & 0xFF; 	//LSB of address

	UCB0CTLW1 = UCASTP_1;
	UCB0TBCNT = data_length; 		//data_length is in words, TBCNT is in bytes, so multiply by 2
	UCB0CTL1  &= ~UCSWRST;
	UCB0CTL1 |= UCTXSTT + UCTR;		//start i2c write operation
	//write the address
	while(!(UCB0IFG & UCTXIFG0));
	UCB0TXBUF = TxAddr[0];
	while(!(UCB0IFG & UCTXIFG0));
	UCB0TXBUF = TxAddr[1];
	//write the data
	for(i = 0; i < data_length; i++)
	{
		while(!(UCB0IFG & UCTXIFG0));
		UCB0TXBUF = write_data[i];
	}

	while(!(UCB0IFG & UCTXIFG0));
	while (!(UCB0IFG & UCBCNTIFG));
	UCB0CTL1 |= UCTXSTP;
	while (!(UCB0IFG & UCSTPIFG));     // Ensure stop condition got sent
	UCB0CTL1  |= UCSWRST;

	PORT_I2C_SEL &= ~(BIT_SCL + BIT_SDA);
	PORT_I2C_OUT |= BIT_SCL + BIT_SDA;
}

void HAL_NFC_INIT()
{
	eUSCIB0_Config();

	//RST RF430 (in case board is still powered but the MSP430 reset for some reason - MSP430 RST button pushed for example)
	PORT_RST_OUT &= ~BIT_RST; 				//RF430CL330H device in reset
	PORT_RST_DIR |= BIT_RST;
	__delay_cycles(1000);
	PORT_RST_OUT |= BIT_RST; 				//release the RF430CL330H

	//configure pin for INTO interrupts
	PORT_INTO_DIR &= ~BIT_INTO; 		//input
	PORT_INTO_OUT |= BIT_INTO; 			//output high for pullup
	PORT_INTO_REN |= BIT_INTO; 			//internal pullup resistor
	PORT_INTO_IFG &= ~BIT_INTO; 		//clear interrupt flag
	PORT_INTO_IES |= BIT_INTO; 			//fire interrupt on high-to-low transition since INTO is setup active low

	__delay_cycles(1000000); 		//leave time for the RF430CL33H to get itself initialized

	while(!(I2C_Read_Register(STATUS_REG) & READY)); //wait until READY bit has been set

	/****************************************************************************/
	/* Configure RF430CL330H for Typical Usage Scenario                         */
	/****************************************************************************/

	//write NDEF memory with Capability Container + NDEF message
	I2C_Write_Continuous(0, NDEF_Application_Data, 76);

	//Enable interrupts for End of Read and End of Write
	I2C_Write_Register(INT_ENABLE_REG, EOW_INT_ENABLE + EOR_INT_ENABLE);

	//Configure INTO pin for active low and enable RF
	I2C_Write_Register(CONTROL_REG, INT_ENABLE + INTO_DRIVE + RF_ENABLE + STANDBY_ENABLE);
	__delay_cycles(1000);

	PORT_INTO_IFG &= ~BIT_INTO; //clear any pending flags
	PORT_INTO_IE |= BIT_INTO;	//enable interrupt
}

void NFC_PROCESS()
{
	uint16_t flags = 0;
	//before we read/write to RF430CL330H we should disable RF
	I2C_Write_Register(CONTROL_REG, INT_ENABLE + INTO_DRIVE); //clear control reg to disable RF
	__delay_cycles(10000);
	flags = I2C_Read_Register(INT_FLAG_REG); //read the flag register to check if a read or write occurred
	__delay_cycles(15);
	I2C_Write_Register(INT_FLAG_REG, EOW_INT_FLAG + EOR_INT_FLAG); //ACK the flags to clear
	__delay_cycles(15);

	if(flags & EOW_INT_FLAG) 			//check if the tag was written
	{
		//tag was updated, so we should read out the new data
		//read out the data
		//Read_Continuous(0, read_data, 200);
		__no_operation(); //breakpoint here to examine the data
	}
	else if(flags & EOR_INT_FLAG) //check if the tag was read
	{
		__no_operation();
		//show that tag was read with LEDs
	}

	flags = 0;

	//Enable interrupts for End of Read and End of Write
	I2C_Write_Register(INT_ENABLE_REG, EOW_INT_ENABLE + EOR_INT_ENABLE);
	__delay_cycles(15);

	//Configure INTO pin for active low and re-enable RF
	I2C_Write_Register(CONTROL_REG, INT_ENABLE + INTO_DRIVE + RF_ENABLE + STANDBY_ENABLE);
	__delay_cycles(15);

	//re-enable INTO
	PORT_INTO_IFG &= ~BIT_INTO;
	PORT_INTO_IE |= BIT_INTO;
}

void NFC_Tag_Update(uint16_t No, uint32_t Sum, tWMRTC RTCData)
{
	int8_t index[] = "0123456789";
	uint8_t i;

	NDEF_Application_Data[35] = 'N';
	NDEF_Application_Data[36] = 'o';
	NDEF_Application_Data[37] = '.';
	NDEF_Application_Data[38] = index[No/1000];
	NDEF_Application_Data[39] = index[No/100];
	NDEF_Application_Data[40] = index[No/10];
	NDEF_Application_Data[41] = index[No%10];
	NDEF_Application_Data[42] = 0x0A;

	NDEF_Application_Data[43] = 'S';
	NDEF_Application_Data[44] = 'u';
	NDEF_Application_Data[45] = 'm';
	NDEF_Application_Data[46] = '=';

	NDEF_Application_Data[47] = index[Sum/10000000];
	NDEF_Application_Data[48] = index[(Sum%10000000)/1000000];
	NDEF_Application_Data[49] = index[(Sum%1000000)/100000];
	NDEF_Application_Data[50] = index[(Sum%100000)/10000];
	NDEF_Application_Data[51] = index[(Sum%10000)/1000];
	NDEF_Application_Data[52] = '.';
	NDEF_Application_Data[53] = index[(Sum%1000)/100];
	NDEF_Application_Data[54] = index[(Sum%100)/10];
	NDEF_Application_Data[55] = index[Sum%10];
	NDEF_Application_Data[56] = 0x0A;

	for(i=47;i<51;i++)
	{
		if(NDEF_Application_Data[i] == '0')  NDEF_Application_Data[i] = ' ';
		else break;
	}

	NDEF_Application_Data[57] = index[2];
	NDEF_Application_Data[58] = index[0];
	NDEF_Application_Data[59] = index[tRTCData.ui8Year/10];
	NDEF_Application_Data[60] = index[tRTCData.ui8Year%10];
	NDEF_Application_Data[61] = '-';
	NDEF_Application_Data[62] = index[tRTCData.ui8Month/10];
	NDEF_Application_Data[63] = index[tRTCData.ui8Month%10];
	NDEF_Application_Data[64] = '-';
	NDEF_Application_Data[65] = index[tRTCData.ui8Day/10];
	NDEF_Application_Data[66] = index[tRTCData.ui8Day%10];
	NDEF_Application_Data[67] = ' ';
	NDEF_Application_Data[68] = index[tRTCData.ui8Hour/10];
	NDEF_Application_Data[69] = index[tRTCData.ui8Hour%10];
	NDEF_Application_Data[70] = ':';
	NDEF_Application_Data[71] = index[tRTCData.ui8Minute/10];
	NDEF_Application_Data[72] = index[tRTCData.ui8Minute%10];
	NDEF_Application_Data[73] = ':';
	NDEF_Application_Data[74] = index[tRTCData.ui8Second/10];
	NDEF_Application_Data[75] = index[tRTCData.ui8Second%10];


	while(!(I2C_Read_Register(STATUS_REG) & READY)); //wait until READY bit has been set
	I2C_Write_Continuous(0, NDEF_Application_Data, 76);
}

#endif
