//*****************************************************************************
//! msp_wm430fr4_rs485.c
//
//! Functions to support wired AMR.
//!
//!
//
//! \author Texas Instruments
//! \author A0222632
//
//! \version Release 1
//
//*****************************************************************************

#include <msp_wm430fr4_golbal.h>

#ifdef Comm_RS485

#define TXBUF_SIZE 30
uint8_t ui8TX_buffer[TXBUF_SIZE];
uint8_t ui8TX_counter;
uint8_t ui8TX_number = TXBUF_SIZE;
#define RXBUF_SIZE 30
uint8_t ui8RX_buffer[RXBUF_SIZE];
uint8_t ui8RX_counter;
bool bRX_enable;
bool bRX_finished;
tWMSN tWMeter_SN = {0x01,0x02,0x03,0x04,0x05,0x06,0x07}; // Default SN, can be set or reset by PC
tWMSN tBroadcast_SN = {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA};
tAMRPacket tSendback;

//*****************************************************************************
//
//! Perform eUSCI initialize.
//!
//
//! \no param
//!
//! \return None.
//
//*****************************************************************************
void eUSCIA0_Config()
{
	UART_PORT_SEL |= UART_BIT_TXD + UART_BIT_RXD; // set pin as UART function

	UCA0CTLW0 |= UCSWRST;                         // Put eUSCI in reset
	UCA0CTLW0 |= UCSSEL__SMCLK;   // 8-e-1 LSB SMCLK

	UCA0BR0 = 6;                                  // 9600 baud
	UCA0BR1 = 0;
	UCA0MCTLW = 0x2200 + UCOS16 + UCBRF_13;       // 1.04MHz/9600

	UCA0CTLW0 &= ~UCSWRST;                        // Initialize eUSCI
	UCA0IE |= UCRXIE;
}

//*****************************************************************************
//
//! Perform Send a frame.
//!
//
//! \param *pPtr Length
//!
//! \return 0 or 1 .
//
//*****************************************************************************
uint8_t Uart_SendFrame(uint8_t *pPtr, uint8_t ui8Length)
{
	uint8_t ui8counter;
	if(UCA0IE & UCTXIE) return 0;                // Last frame hasn't been sent out
	if(ui8Length > TXBUF_SIZE) return 0;            // Length overflow the buffer
	for(ui8counter=0;ui8counter<ui8Length;ui8counter++)      // Load the data to buffer
	{
		ui8TX_buffer[ui8counter] = pPtr[ui8counter];
	}
	ui8TX_counter = 0;                               // Clear transfer counter
	ui8TX_number = ui8Length;                           // Number of bytes to be sent
	UCA0IFG |= UCTXIFG;                           // Set an interruput manually
	UCA0IE |= UCTXIE;                             // Enable transfer interrupt
	return 1;                                     // Send successfully
}

//*****************************************************************************
//
//! Perform SN check.
//!
//
//! \param tA tB
//!
//! \return false or true .
//
//*****************************************************************************
bool SN_Check(tWMSN tA, tWMSN tB)
{
	if(tA.ui8Addr0 != tB.ui8Addr0) return false;
	if(tA.ui8Addr1 != tB.ui8Addr1) return false;
	if(tA.ui8Addr2 != tB.ui8Addr2) return false;
	if(tA.ui8Addr3 != tB.ui8Addr3) return false;
	if(tA.ui8Addr4 != tB.ui8Addr4) return false;
	if(tA.ui8Addr5 != tB.ui8Addr5) return false;
	if(tA.ui8Addr6 != tB.ui8Addr6) return false;
	else return true;
}

//*****************************************************************************
//
//! Perform Uart data process.
//!
//
//! \param None
//!
//! \return None
//
//*****************************************************************************
uint8_t Uart_DataProcess()
{
	tAMRPacket tDataget;
	uint8_t *pDataget = &tDataget.ui8SOF;
	uint8_t ui8counter = 0;
	uint8_t ui8temp = 0;
	tDataget.ui8SOF = ui8RX_buffer[ui8counter++];
	tDataget.ui8MeterType = ui8RX_buffer[ui8counter++];
	tDataget.pWMAddr.ui8Addr0 = ui8RX_buffer[ui8counter++];
	tDataget.pWMAddr.ui8Addr1 = ui8RX_buffer[ui8counter++];
	tDataget.pWMAddr.ui8Addr2 = ui8RX_buffer[ui8counter++];
	tDataget.pWMAddr.ui8Addr3 = ui8RX_buffer[ui8counter++];
	tDataget.pWMAddr.ui8Addr4 = ui8RX_buffer[ui8counter++];
	tDataget.pWMAddr.ui8Addr5 = ui8RX_buffer[ui8counter++];
	tDataget.pWMAddr.ui8Addr6 = ui8RX_buffer[ui8counter++];
	tDataget.ui8Command = ui8RX_buffer[ui8counter++];
	tDataget.ui8DataLength = ui8RX_buffer[ui8counter++];
	for(ui8temp = 0; ui8temp < 12; ui8temp++)
	{
		tDataget.Data[ui8temp] = ui8RX_buffer[ui8counter++];
	}
	tDataget.ui8Checksum = ui8RX_buffer[ui8counter++];
	tDataget.ui8EOF = ui8RX_buffer[ui8counter++];

	if(tDataget.ui8MeterType != WM_Cold)
	{
		LCD_Display_Error(Error_MT);
		return Error_MT;
	}
	if((SN_Check(tDataget.pWMAddr,tBroadcast_SN)==false) && (SN_Check(tDataget.pWMAddr,tWMeter_SN))==false)
	{
		LCD_Display_Error(Error_Address);
		return Error_Address;
	}
	if((tDataget.ui8Command)&0x80)
	{
		LCD_Display_Error(Error_CMD);
		return Error_CMD;
	}

	switch((tDataget.ui8Command)&0x3F)
	{
	case CMD_ReadData:
	{
		tDataget.ui8Command |= 0x80;
		tDataget.ui8DataLength = 0x04;
		tDataget.Data[0] = (uint8_t)(ui32Sum&0x000000FF);
		tDataget.Data[1] = (uint8_t)((ui32Sum&0x0000FF00)>>8);
		tDataget.Data[2] = (uint8_t)((ui32Sum&0x00FF0000)>>16);
		tDataget.Data[3] = (uint8_t)((ui32Sum&0xFF000000)>>24);
		tDataget.Data[4] = (uint8_t)(ui16Freq&0x00FF);
		tDataget.Data[5] = (uint8_t)((ui16Freq&0xFF00)>>8);
		Uart_SendFrame(pDataget,25);
	}
	break;
	case CMD_WriteData:
	{
		ui32Sum = (uint16_t)(tDataget.Data[0] + (tDataget.Data[1]<<8));
	}
	break;
	case CMD_ReadKeyVer:
		tDataget.ui8Command |= 0x80;
		tDataget.ui8DataLength = 0x01;
		tDataget.Data[0] = Firmware_Version;
		tDataget.Data[1] = 0;
		tDataget.Data[2] = 0;
		tDataget.Data[3] = 0;
		tDataget.Data[4] = 0;
		tDataget.Data[5] = 0;
		tDataget.Data[6] = 0;
		tDataget.Data[7] = 0;
		Uart_SendFrame(pDataget,25);
		break;
	case CMD_ReadAddr:
	{
		tDataget.ui8Command |= 0x80;
		tDataget.ui8DataLength = 0x07;
		tDataget.Data[0] = tWMeter_SN.ui8Addr0;
		tDataget.Data[1] = tWMeter_SN.ui8Addr1;
		tDataget.Data[2] = tWMeter_SN.ui8Addr2;
		tDataget.Data[3] = tWMeter_SN.ui8Addr3;
		tDataget.Data[4] = tWMeter_SN.ui8Addr4;
		tDataget.Data[5] = tWMeter_SN.ui8Addr5;
		tDataget.Data[6] = tWMeter_SN.ui8Addr6;
		Uart_SendFrame(pDataget,25);
	}
	break;
	case CMD_WriteAddr:
	{
		tWMeter_SN.ui8Addr0 = tDataget.Data[0]; // Update new SN
		tWMeter_SN.ui8Addr1 = tDataget.Data[1];
		tWMeter_SN.ui8Addr2 = tDataget.Data[2];
		tWMeter_SN.ui8Addr3 = tDataget.Data[3];
		tWMeter_SN.ui8Addr4 = tDataget.Data[4];
		tWMeter_SN.ui8Addr5 = tDataget.Data[5];
		tWMeter_SN.ui8Addr6 = tDataget.Data[6];
	}
	break;
	case CMD_Calibrate:
	{
		ui32Sum = (uint16_t)(tDataget.Data[0] + (tDataget.Data[1]<<8));
	}
	break;
	case CMD_ReadRTC:
	{
		tDataget.ui8Command |= 0x80;
		tDataget.ui8DataLength = 0x06;
		tDataget.Data[0] = tRTCData.ui8Second;
		tDataget.Data[1] = tRTCData.ui8Minute;
		tDataget.Data[2] = tRTCData.ui8Hour;
		tDataget.Data[3] = tRTCData.ui8Day;
		tDataget.Data[4] = tRTCData.ui8Month;
		tDataget.Data[5] = tRTCData.ui8Year;
		Uart_SendFrame(pDataget,25);
	}
	break;
	case CMD_WriteRTC:
	{
		tRTCData.ui8Second = tDataget.Data[0];
		tRTCData.ui8Minute = tDataget.Data[1];
		tRTCData.ui8Hour = tDataget.Data[2];
		tRTCData.ui8Day = tDataget.Data[3];
		tRTCData.ui8Month = tDataget.Data[4];
		tRTCData.ui8Year = tDataget.Data[5];
	}
	break;
	case CMD_OpenValve:
	{
		bOpen_Valve_Flag = true;
	}
	break;
	case CMD_CloseValve:
	{
		bClose_Valve_Flag = true;
	}
	break;
	}
	return (1);
}


void HAL_RS485_INIT()
{
	eUSCIA0_Config();
}


//#pragma vector=USCI_A0_VECTOR
//__interrupt void USCI_A0_ISR(void)
//{
//	switch(__even_in_range(UCA0IV,USCI_UART_UCTXCPTIFG))
//	{
//	case USCI_NONE: break;
//	case USCI_UART_UCRXIFG:
//		if(UCA0RXBUF == 0x55)//start upload data
//		{
//			bRS485_Send_Flag = true;
//		}
//		else if(UCA0RXBUF == 0xAA)//stop upload
//		{
//			bRS485_Send_Flag = false;
//		}
//		else if(UCA0RXBUF == 0xBB)//open valve
//		{
//			bOpen_Valve_Flag = true;
//		}
//		else if(UCA0RXBUF == 0xDD)//close valve
//		{
//			bClose_Valve_Flag = true;
//		}
//		break;
//	case USCI_UART_UCTXIFG: break;
//	case USCI_UART_UCSTTIFG: break;
//	case USCI_UART_UCTXCPTIFG: break;
//	}
//}


#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR(void)
{
	switch(__even_in_range(UCA0IV,USCI_UART_UCTXCPTIFG))
	{
	case USCI_NONE: break;
	case USCI_UART_UCRXIFG:
	{
		uint8_t ui8RX_temp;
		ui8RX_temp = UCA0RXBUF;
		if(!bRX_finished)
		{
			if(bRX_enable)
			{
				ui8RX_buffer[ui8RX_counter] = ui8RX_temp;
				if((ui8RX_temp == 0x16) && (ui8RX_counter > 10+ui8RX_buffer[10]))
				{
					bRX_enable = false;
					bRX_finished = true;
				}
				else ui8RX_counter++;
			}
			else if(ui8RX_temp == 0x68)
			{
				bRX_enable = true;
				ui8RX_counter = 0;
				ui8RX_buffer[ui8RX_counter++] = ui8RX_temp;
			}
		}
	}
		break;
	case USCI_UART_UCTXIFG:
		UCA0TXBUF = ui8TX_buffer[ui8TX_counter];
		ui8TX_counter++;
		if(ui8TX_counter>=ui8TX_number)
		{
			UCA0IE &= ~UCTXIE;
		}
		break;
	case USCI_UART_UCSTTIFG: break;
	case USCI_UART_UCTXCPTIFG: break;
	}
}

#endif

